@@ -24,7 +24,7 @@ HTTP_METHODS = (
     'TRACE',
 )
 
-DEFAULT_MEDIA_TYPE = 'application/json; charset=utf-8'
+DEFAULT_MEDIA_TYPE = 'application/json; charset=UTF-8'
 
 
 # Hoist classes and functions into the falcon namespace
@@ -32,7 +32,9 @@ from falcon.version import __version__  # NOQA
 from falcon.api import API, DEFAULT_MEDIA_TYPE  # NOQA
 from falcon.status_codes import *  # NOQA
 from falcon.errors import *  # NOQA
+from falcon.redirects import *  # NOQA
 from falcon.http_error import HTTPError  # NOQA
+from falcon.http_status import HTTPStatus  # NOQA
 from falcon.util import *  # NOQA
 from falcon.hooks import before, after  # NOQA
 from falcon.request import Request, RequestOptions  # NOQA
