@@ -50,18 +50,19 @@ class HTTPUnauthorized(HTTPError):
         title (str): Error title (e.g., 'Authentication Required').
         description (str): Human-friendly description of the error, along with
             a helpful suggestion or two.
-        scheme (str): Authentication scheme to use as the value of the
-            WWW-Authenticate header in the response (default ``None``).
+        challenges (iterable of str): One or more authentication
+            challenges to use as the value of the WWW-Authenticate header in
+            the response. See also:
+            http://tools.ietf.org/html/rfc7235#section-2.1
         kwargs (optional): Same as for ``HTTPError``.
 
     """
 
-    def __init__(self, title, description, **kwargs):
+    def __init__(self, title, description, challenges, **kwargs):
         headers = kwargs.setdefault('headers', {})
 
-        scheme = kwargs.pop('scheme', None)
-        if scheme is not None:
-            headers['WWW-Authenticate'] = scheme
+        if challenges:
+            headers['WWW-Authenticate'] = ', '.join(challenges)
 
         super(HTTPUnauthorized, self).__init__(status.HTTP_401, title,
                                                description, **kwargs)
@@ -252,7 +253,7 @@ class HTTPRequestEntityTooLarge(HTTPError):
         retry_after (datetime or int, optional): Value for the Retry-After
             header. If a ``datetime`` object, will serialize as an HTTP date.
             Otherwise, a non-negative ``int`` is expected, representing the
-            number of seconds to wait. See also: http://goo.gl/DIrWr .
+            number of seconds to wait.
         kwargs (optional): Same as for ``HTTPError``.
 
     """
@@ -262,7 +263,6 @@ class HTTPRequestEntityTooLarge(HTTPError):
 
         if isinstance(retry_after, datetime):
             headers['Retry-After'] = util.dt_to_http(retry_after)
-
         elif retry_after is not None:
             headers['Retry-After'] = str(retry_after)
 
@@ -306,6 +306,82 @@ class HTTPRangeNotSatisfiable(NoRepresentation, HTTPError):
                                                       headers=headers)
 
 
+class HTTPUnprocessableEntity(HTTPError):
+    """422 Unprocessable Entity.
+
+    The request was well-formed but was unable to be followed due to semantic
+    errors. See also: http://www.ietf.org/rfc/rfc4918.
+
+    Args:
+        title (str): Error title (e.g., 'Missing title field').
+        description (str): Human-friendly description of the error, along with
+            a helpful suggestion or two.
+        kwargs (optional): Same as for ``HTTPError``.
+    """
+
+    def __init__(self, title, description, **kwargs):
+        super(HTTPUnprocessableEntity, self).__init__(status.HTTP_422, title,
+                                                      description, **kwargs)
+
+
+class HTTPTooManyRequests(HTTPError):
+    """429 Too Many Requests.
+
+    The user has sent too many requests in a given amount of time
+    ("rate limiting").
+
+    The response representations SHOULD include details explaining the
+    condition, and MAY include a Retry-After header indicating how long
+    to wait before making a new request.
+
+    (RFC 6585)
+
+    Args:
+        title (str): Error title (e.g., 'Too Many Requests').
+        description (str): Human-friendly description of the rate limit that
+            was exceeded.
+        retry_after (datetime or int, optional): Value for the Retry-After
+            header. If a ``datetime`` object, will serialize as an HTTP date.
+            Otherwise, a non-negative ``int`` is expected, representing the
+            number of seconds to wait.
+        kwargs (optional): Same as for ``HTTPError``.
+
+    """
+
+    def __init__(self, title, description, retry_after=None, **kwargs):
+        headers = kwargs.setdefault('headers', {})
+
+        if isinstance(retry_after, datetime):
+            headers['Retry-After'] = util.dt_to_http(retry_after)
+        elif retry_after is not None:
+            headers['Retry-After'] = str(retry_after)
+
+        super(HTTPTooManyRequests, self).__init__(status.HTTP_429,
+                                                  title,
+                                                  description,
+                                                  **kwargs)
+
+
+class HTTPUnavailableForLegalReasons(OptionalRepresentation, HTTPError):
+    """451 Unavailable For Legal Reasons.
+
+    This status code indicates that the server is denying access to the
+    resource as a consequence of a legal demand.
+
+    See also:
+    https://datatracker.ietf.org/doc/draft-ietf-httpbis-legally-restricted-status/
+
+    Args:
+        title (str): Error title (e.g., 'Legal reason: <reason>').
+        kwargs (optional): Same as for ``HTTPError``.
+
+    """
+
+    def __init__(self, title, **kwargs):
+        super(HTTPUnavailableForLegalReasons, self).__init__(status.HTTP_451,
+                                                             title, **kwargs)
+
+
 class HTTPInternalServerError(HTTPError):
     """500 Internal Server Error.
 
@@ -349,7 +425,7 @@ class HTTPServiceUnavailable(HTTPError):
         retry_after (datetime or int): Value for the Retry-After header. If a
             ``datetime`` object, will serialize as an HTTP date. Otherwise,
             a non-negative ``int`` is expected, representing the number of
-            seconds to wait. See also: http://goo.gl/DIrWr .
+            seconds to wait.
         kwargs (optional): Same as for ``HTTPError``.
 
     """
