@@ -12,20 +12,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from falcon.errors import HTTPBadRequest
+from falcon.errors import HTTPNotFound
 from falcon.status_codes import HTTP_204
-from falcon.status_codes import HTTP_400
-from falcon.status_codes import HTTP_404
 from falcon.status_codes import HTTP_405
 
 
 def path_not_found(req, resp, **kwargs):
-    """Simply sets responseto "404 Not Found", no body."""
-    resp.status = HTTP_404
+    """Raise 404 HTTPNotFound error"""
+    raise HTTPNotFound()
 
 
 def bad_request(req, resp, **kwargs):
-    """Sets response to "400 Bad Request", no body."""
-    resp.status = HTTP_400
+    """Raise 400 HTTPBadRequest error"""
+    raise HTTPBadRequest('Bad request', 'Invalid HTTP method')
 
 
 def create_method_not_allowed(allowed_methods):
