@@ -3,6 +3,7 @@
 from datetime import datetime
 import functools
 import io
+import json
 import random
 import sys
 
@@ -10,7 +11,7 @@ import testtools
 import six
 
 import falcon
-import falcon.testing
+from falcon import testing
 from falcon import util
 from falcon.util import uri
 
@@ -79,6 +80,32 @@ class TestFalconUtils(testtools.TestCase):
             falcon.http_date_to_dt('Thu, 04 Apr 2013 10:28:54 GMT'),
             datetime(2013, 4, 4, 10, 28, 54))
 
+        self.assertRaises(
+            ValueError,
+            falcon.http_date_to_dt, 'Thu, 04-Apr-2013 10:28:54 GMT')
+
+        self.assertEqual(
+            falcon.http_date_to_dt('Thu, 04-Apr-2013 10:28:54 GMT',
+                                   obs_date=True),
+            datetime(2013, 4, 4, 10, 28, 54))
+
+        self.assertRaises(
+            ValueError,
+            falcon.http_date_to_dt, 'Sun Nov  6 08:49:37 1994')
+
+        self.assertRaises(
+            ValueError,
+            falcon.http_date_to_dt, 'Nov  6 08:49:37 1994', obs_date=True)
+
+        self.assertEqual(
+            falcon.http_date_to_dt('Sun Nov  6 08:49:37 1994', obs_date=True),
+            datetime(1994, 11, 6, 8, 49, 37))
+
+        self.assertEqual(
+            falcon.http_date_to_dt('Sunday, 06-Nov-94 08:49:37 GMT',
+                                   obs_date=True),
+            datetime(1994, 11, 6, 8, 49, 37))
+
     def test_pack_query_params_none(self):
         self.assertEqual(
             falcon.to_query_str({}),
@@ -178,7 +205,7 @@ class TestFalconUtils(testtools.TestCase):
 
     def test_prop_uri_encode_value_models_stdlib_quote_safe_tilde(self):
         equiv_quote = functools.partial(
-            six.moves.urllib.parse.quote, safe="~"
+            six.moves.urllib.parse.quote, safe='~'
         )
         for case in self.uris:
             expect = equiv_quote(case)
@@ -196,14 +223,14 @@ class TestFalconUtils(testtools.TestCase):
 
     def test_parse_query_string(self):
         query_strinq = (
-            "a=http%3A%2F%2Ffalconframework.org%3Ftest%3D1"
-            "&b=%7B%22test1%22%3A%20%22data1%22%"
-            "2C%20%22test2%22%3A%20%22data2%22%7D"
-            "&c=1,2,3"
-            "&d=test"
-            "&e=a,,%26%3D%2C"
-            "&f=a&f=a%3Db"
-            "&%C3%A9=a%3Db"
+            'a=http%3A%2F%2Ffalconframework.org%3Ftest%3D1'
+            '&b=%7B%22test1%22%3A%20%22data1%22%'
+            '2C%20%22test2%22%3A%20%22data2%22%7D'
+            '&c=1,2,3'
+            '&d=test'
+            '&e=a,,%26%3D%2C'
+            '&f=a&f=a%3Db'
+            '&%C3%A9=a%3Db'
         )
         decoded_url = 'http://falconframework.org?test=1'
         decoded_json = '{"test1": "data1", "test2": "data2"}'
@@ -271,25 +298,25 @@ class TestFalconUtils(testtools.TestCase):
                          ('falcon.example.com', 42))
 
 
-class TestFalconTesting(falcon.testing.TestBase):
+class TestFalconTesting(testing.TestBase):
     """Catch some uncommon branches not covered elsewhere."""
 
     def test_path_escape_chars_in_create_environ(self):
-        env = falcon.testing.create_environ('/hello%20world%21')
+        env = testing.create_environ('/hello%20world%21')
         self.assertEqual(env['PATH_INFO'], '/hello world!')
 
     def test_unicode_path_in_create_environ(self):
         if six.PY3:
             self.skip('Test does not apply to Py3K')
 
-        env = falcon.testing.create_environ(u'/fancy/unícode')
+        env = testing.create_environ(u'/fancy/unícode')
         self.assertEqual(env['PATH_INFO'], '/fancy/un\xc3\xadcode')
 
-        env = falcon.testing.create_environ(u'/simple')
+        env = testing.create_environ(u'/simple')
         self.assertEqual(env['PATH_INFO'], '/simple')
 
     def test_none_header_value_in_create_environ(self):
-        env = falcon.testing.create_environ('/', headers={'X-Foo': None})
+        env = testing.create_environ('/', headers={'X-Foo': None})
         self.assertEqual(env['HTTP_X_FOO'], '')
 
     def test_decode_empty_result(self):
@@ -297,4 +324,69 @@ class TestFalconTesting(falcon.testing.TestBase):
         self.assertEqual(body, '')
 
     def test_httpnow_alias_for_backwards_compat(self):
-        self.assertIs(falcon.testing.httpnow, util.http_now)
+        self.assertIs(testing.httpnow, util.http_now)
+
+
+class TestFalconTestCase(testing.TestCase):
+    """Verify some branches not covered elsewhere."""
+
+    def test_status(self):
+        resource = testing.SimpleTestResource(status=falcon.HTTP_702)
+        self.api.add_route('/', resource)
+
+        result = self.simulate_get()
+        self.assertEqual(result.status, falcon.HTTP_702)
+
+    def test_wsgi_iterable_not_closeable(self):
+        result = testing.Result([], falcon.HTTP_200, [])
+        self.assertFalse(result.content)
+
+    def test_path_must_start_with_slash(self):
+        self.assertRaises(ValueError, self.simulate_get, 'foo')
+
+    def test_cached_text_in_result(self):
+        self.api.add_route('/', testing.SimpleTestResource(body='test'))
+
+        result = self.simulate_get()
+        self.assertEqual(result.text, result.text)
+
+    def test_simple_resource_body_json_xor(self):
+        self.assertRaises(
+            ValueError,
+            testing.SimpleTestResource,
+            body='',
+            json={},
+        )
+
+    def test_query_string(self):
+        class SomeResource(object):
+            def on_get(self, req, resp):
+                doc = {}
+
+                doc['oid'] = req.get_param_as_int('oid')
+                doc['detailed'] = req.get_param_as_bool('detailed')
+
+                resp.body = json.dumps(doc)
+
+        self.api.add_route('/', SomeResource())
+
+        result = self.simulate_get(query_string='oid=42&detailed=no')
+        self.assertEqual(result.json['oid'], 42)
+        self.assertFalse(result.json['detailed'])
+
+    def test_query_string_no_question(self):
+        self.assertRaises(ValueError, self.simulate_get, query_string='?x=1')
+
+    def test_query_string_in_path(self):
+        self.assertRaises(ValueError, self.simulate_get, path='/thing?x=1')
+
+
+class FancyAPI(falcon.API):
+    pass
+
+
+class FancyTestCase(testing.TestCase):
+    api_class = FancyAPI
+
+    def test_something(self):
+        self.assertTrue(isinstance(self.api, FancyAPI))
