@@ -4,5 +4,5 @@ if [ "$JYTHON" = "true" ]; then
     # Smoke test
     $HOME/jython/bin/jython falcon/bench/bench.py -t 1 -b falcon falcon-ext
 else
-    tox -e $TOX_ENV
+    tox
 fi
